@@ -45,6 +45,7 @@
 
 volatile int keep_going = 1;
 int socket_fd;
+char socket_name[64];
 int one_shot_mode;
 int debug_mode;
 int foreground_mode;
@@ -57,7 +58,6 @@ unsigned long power_thresh = ULONG_MAX;
 unsigned long deepest_cache = 2;
 unsigned long long cycle_count = 0;
 char *pidfile = NULL;
-char *banscript = NULL;
 char *polscript = NULL;
 long HZ;
 int sleep_interval = SLEEP_INTERVAL;
@@ -87,7 +87,6 @@ struct option lopts[] = {
 	{"hintpolicy", 1, NULL, 'h'},
 	{"powerthresh", 1, NULL, 'p'},
 	{"banirq", 1 , NULL, 'i'},
-	{"banscript", 1, NULL, 'b'},
 	{"deepestcache", 1, NULL, 'c'},
 	{"policyscript", 1, NULL, 'l'},
 	{"pid", 1, NULL, 's'},
@@ -117,7 +116,7 @@ static void parse_command_line(int argc, char **argv)
 	unsigned long val;
 
 	while ((opt = getopt_long(argc, argv,
-		"odfji:p:s:c:b:l:m:t:V",
+		"odfji:p:s:c:l:m:t:V",
 		lopts, &longind)) != -1) {
 
 		switch(opt) {
@@ -129,18 +128,6 @@ static void parse_command_line(int argc, char **argv)
 				version();
 				exit(1);
 				break;
-			case 'b':
-#ifndef INCLUDE_BANSCRIPT
-				/*
-				 * Banscript is no longer supported unless
-				 * explicitly enabled
-				 */
-				log(TO_CONSOLE, LOG_INFO, "--banscript is not supported on this version of irqbalance, please use --policyscript\n");
-				usage();
-				exit(1);
-#endif
-				banscript = strdup(optarg);
-				break;
 			case 'c':
 				deepest_cache = strtoul(optarg, NULL, 10);
 				if (deepest_cache == ULONG_MAX || deepest_cache < 1) {
@@ -270,7 +257,6 @@ gboolean scan(gpointer data)
 	log(TO_CONSOLE, LOG_INFO, "\n\n\n-----------------------------------------------------------------------------\n");
 	clear_work_stats();
 	parse_proc_interrupts();
-	parse_proc_stat();
 
 
 	/* cope with cpu hotplug -- detected during /proc/interrupts parsing */
@@ -288,8 +274,9 @@ gboolean scan(gpointer data)
 		sleep_approx(sleep_interval);
 		clear_work_stats();
 		parse_proc_interrupts();
-		parse_proc_stat();
-	} 
+	}
+
+	parse_proc_stat();
 
 	if (cycle_count)	
 		update_migration_status();
@@ -385,11 +372,11 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 		sock = accept(fd, NULL, NULL);
 		if (sock < 0) {
 			log(TO_ALL, LOG_WARNING, "Connection couldn't be accepted.\n");
-			return TRUE;
+			goto out;
 		}
 		if ((recv_size = recvmsg(sock, &msg, 0)) < 0) {
 			log(TO_ALL, LOG_WARNING, "Error while receiving data.\n");
-			return TRUE;
+			goto out;
 		}
 		cmsg = CMSG_FIRSTHDR(&msg);
 		if ((cmsg->cmsg_level == SOL_SOCKET) &&
@@ -401,7 +388,7 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 		}
 		if (!valid_user) {
 			log(TO_ALL, LOG_INFO, "Permission denied for user to connect to socket.\n");
-			return TRUE;
+			goto out;
 		}
 
 		if (!strncmp(buff, "stats", strlen("stats"))) {
@@ -421,6 +408,7 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 				if (new_iterval >= 1) {
 					sleep_interval = new_iterval;
 				}
+				free(sleep_string);
 			} else if (!(strncmp(buff + strlen("settings "), "ban irqs ",
 							strlen("ban irqs ")))) {
 				char *end;
@@ -432,12 +420,14 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 				cl_banned_irqs = NULL;
 				need_rescan = 1;
 				if (!strncmp(irq_string, "NONE", strlen("NONE"))) {
-					return TRUE;
+					free(irq_string);
+					goto out;
 				}
 				int irq = strtoul(irq_string, &end, 10);
 				do {
 					add_cl_banned_irq(irq);
 				} while((irq = strtoul(end, &end, 10)));
+				free(irq_string);
 			} else if (!(strncmp(buff + strlen("settings "), "cpus ",
 							strlen("cpus")))) {
 				char *cpu_ban_string = malloc(
@@ -449,12 +439,13 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 					banned_cpumask_from_ui = NULL;
 				}
 				need_rescan = 1;
+				free(cpu_ban_string);
 			}
 		}
 		if (!strncmp(buff, "setup", strlen("setup"))) {
 			char banned[512];
-			char *setup = calloc(strlen("SLEEP  ") + 11 +1, 1);
-			snprintf(setup, 2048, "SLEEP %d ", sleep_interval);
+			char *setup = calloc(strlen("SLEEP  ") + 11 + 1, 1);
+			snprintf(setup, strlen("SLEEP  ") + 11 + 1, "SLEEP %d ", sleep_interval);
 			if(g_list_length(cl_banned_irqs) > 0) {
 				for_each_irq(cl_banned_irqs, get_irq_data, setup);
 			}
@@ -463,14 +454,18 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 			snprintf(setup + strlen(setup), strlen(banned) + 7 + 1,
 					"BANNED %s", banned);
 			send(sock, setup, strlen(setup), 0);
+			free(setup);
 		}
 
 		close(sock);
 	}
+
+out:
+	free(msg.msg_control);
 	return TRUE;
 }
 
-int init_socket(char *socket_name)
+int init_socket()
 {
 	struct sockaddr_un addr;
 	memset(&addr, 0, sizeof(struct sockaddr_un));
@@ -481,14 +476,25 @@ int init_socket(char *socket_name)
 		return 1;
 	}
 
+	/*
+	 * First try to create a file-based socket in tmpfs.  If that doesn't
+	 * succeed, fall back to an abstract socket (non file-based).
+	 */
 	addr.sun_family = AF_UNIX;
-	addr.sun_path[0] = '\0';
-	strncpy(addr.sun_path + 1, socket_name, strlen(socket_name));
-	if (bind(socket_fd, (struct sockaddr *)&addr,
-				sizeof(sa_family_t) + strlen(socket_name) + 1) < 0) {
-		log(TO_ALL, LOG_WARNING, "Daemon couldn't be bound to the socket.\n");
-		return 1;
+	snprintf(socket_name, 64, "%s/%s%d.sock", SOCKET_TMPFS, SOCKET_PATH, getpid());
+	strncpy(addr.sun_path, socket_name, strlen(socket_name));
+	if (bind(socket_fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+		log(TO_ALL, LOG_WARNING, "Daemon couldn't be bound to the file-based socket.\n");
+
+		/* Try binding to abstract */
+		memset(&addr, 0, sizeof(struct sockaddr_un));
+		addr.sun_family = AF_UNIX;
+		if (bind(socket_fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+			log(TO_ALL, LOG_WARNING, "Daemon couldn't be bound to the abstract socket, bailing out.\n");
+			return 1;
+		}
 	}
+
 	int optval = 1;
 	if (setsockopt(socket_fd, SOL_SOCKET, SO_PASSCRED, &optval, sizeof(optval)) < 0) {
 		log(TO_ALL, LOG_WARNING, "Unable to set socket options.\n");
@@ -560,11 +566,6 @@ int main(int argc, char** argv)
 	if (geteuid() != 0)
 		log(TO_ALL, LOG_WARNING, "Irqbalance hasn't been executed under root privileges, thus it won't in fact balance interrupts.\n");
 
-	if (banscript) {
-		char *note = "Please note that --banscript is deprecated, please use --policyscript instead";
-		log(TO_ALL, LOG_WARNING, "%s\n", note);
-	}
-
 	HZ = sysconf(_SC_CLK_TCK);
 	if (HZ == -1) {
 		log(TO_ALL, LOG_WARNING, "Unable to determin HZ defaulting to 100\n");
@@ -618,10 +619,7 @@ int main(int argc, char** argv)
 	parse_proc_interrupts();
 	parse_proc_stat();
 
-	char socket_name[64];
-	snprintf(socket_name, 64, "%s%d.sock", SOCKET_PATH, getpid());
-
-	if (init_socket(socket_name)) {
+	if (init_socket()) {
 		return EXIT_FAILURE;
 	}
 	main_loop = g_main_loop_new(NULL, FALSE);
@@ -639,6 +637,8 @@ int main(int argc, char** argv)
 		unlink(pidfile);
 	/* Remove socket */
 	close(socket_fd);
+	if (socket_name[0])
+		unlink(socket_name);
 
 	return EXIT_SUCCESS;
 }
